<?php

namespace Drupal\ustudio;

use GuzzleHttp\Client;
use stdClass;

class Ustudio {

  private $endpoint = 'https://app.ustudio.com/api/v2/';
  private $root;
  private $accountID;
  private $accountToken;
  private $cacheEnabled;
  private $cacheBin = 'cache';
  private $cacheIdPrefix = 'ustudio:';
  private $cacheTTL = 30; // cache 30 seconds

  /** @var Client $client */
  private $client;

  public function __construct($accountID, $accountToken, Client $client = NULL) {
    $this->accountID = $accountID;
    $this->accountToken = $accountToken;
    $this->root = $this->endpoint . 'studios/' . $accountID . '/';
    $this->client = NULL !== $client ? $client : new Client();
    $this->cacheEnabled = function_exists('cache_get');
  }

  /**
   * Get ustudio endpoint.
   *
   * @param string $path
   * @return string
   */
  private function getEndpoint($path) {
    $url = rtrim($this->root, '/');
    if (!empty($path)) {
      $url .= '/' . $path;
    }
    return $url . '?token=' . $this->accountToken;
  }

  /**
   * Helper method to perform request to ustudio.
   *
   * @param string $method
   * @param string $endpoint
   * @param array $params
   * @return stdClass
   */
  private function request($method, $endpoint, $params = array()) {
    if ($this->cacheEnabled && ('GET' === $method)) {
      if ($cache = $this->getRequestCache($endpoint, $params)) {
        return $cache;
      }
    }

    $json = $this
      ->client
      ->{$method}($this->getEndpoint($endpoint), $params)
      ->getBody()
      ->getContents()
    ;
    $return = json_decode($json);

    if ($this->cacheEnabled && ('GET' === $method)) {
      $this->setRequestCache($endpoint, $params, $return);
    }

    return $return;
  }

  private function getCacheId($endpoint, $params) {
    return $this->cacheIdPrefix . substr(md5($endpoint . print_r($params, TRUE)), 0, 15);
  }

  private function setRequestCache($endpoint, $params, $return) {
    $cacheId = $this->getCacheId($endpoint, $params);
    return cache_set($this->cacheIdPrefix . $cacheId, $return, $this->cacheBin, time() + $this->cacheTTL);
  }

  private function getRequestCache($endpoint, $params) {
    $cacheId = $this->getCacheId($endpoint, $params);
    $cache = cache_get($cacheId, $this->cacheBin);
    if ($cache) {
      return $cache->data;
    }
    return FALSE;
  }

  public function getInfo() {
    return $this->request('GET', '');
  }

  public function getMembers() {
    return $this->request('GET', 'members');
  }

  /**
   * Get videos list
   *
   * @param array $params
   * @return object[] Video objects
   */
  public function getVideos($params = array()) {
    $query = array();

    foreach ($params as $key => $value) {
      switch ($key) {
        case 'after': # Date in format YYYY-MM-DD
        case 'query': # keyword searching
          $query[$key] = strip_tags($value);
          break;
      }
    }

    return $this->request('GET', 'videos', array('query' => $query));
  }

  /**
   * Get video statistics
   *
   * @param string $type One of these options:
   *  vdp:  Views, grouped by videos
   *  ddp:  Views, grouped by destinations
   *  vdi:  Impressions, grouped by videos
   *  ddi:  Impressions, grouped by destinations
   *  vdpc: Engagement, grouped by videos
   *  ddpc: Engagement, grouped by destinations
   *  vde:  …
   *  dde:  …
   * @param string[] $videoIds
   * @param string   $after   Format: YYY-MM-DD
   * @param string   $before  Format: YYY-MM-DD
   */
  public function getStatistics($type, $videoIds, $after, $before) {
    return $this->request('GET', 'statistics', array(
          'query' => array(
              'query'  => $type,
              'videos' => implode(',', $videoIds),
              'after'  => $after,
              'before' => $before,
          )
    ));
  }

  /**
   * Get video details.
   *
   * @param string $videoId ID of video
   * @return object
   */
  public function getVideo($videoId) {
    return $this->request('GET', "videos/{$videoId}");
  }

  /**
   * @param string[] $videoIds
   * @param string $collectionId
   */
  public function addVideoToCollection($videoIds, $collectionId) {
    return $this->request('POST', "collections/{$collectionId}/videos", array('video_uids' => $videoIds));
  }

  /**
   * Get video assets
   *
   * @param $videoId ID of video
   */
  public function getVideoAsset($videoId) {
    return $this->request('GET', "videos/{$videoId}/asset");
  }

  /**
   * Get collections list
   */
  public function getCollections() {
    return $this->request('GET', 'collections');
  }

  public function getCollection($collectionId) {
    return $this->request('GET', "collectoins/{$collectionId}");
  }

  /**
   * Get videos by collection
   *
   * @param string $collectionId
   * @param $params
   */
  public function getCollectionVideos($collectionId, $params) {
    $query = array();
    foreach ($params as $key => $value) {
      switch ($key) {
        case 'after': # Date in format YYYY-MM-DD
        case 'query': # keyword searching
          $query[$key] = strip_tags($value);
          break;
      }
    }
    return $this->request('GET', "collections/{$collectionId}/videos", array('query' => $query));
  }

  public function getDestinations() {
    return $this->request('GET', 'destinations');
  }

  public function getDestination($destinationId) {
    return $this->request('GET', "destinations/{$destinationId}");
  }

  public function getDestinationTemplates() {
    return $this->request('GET', 'destination_templates');
  }

  public function getDestinationTemplate($id) {
    return $this->request('GET', "destination_templates/{$id}");
  }

}
